import numpy as np
import torch
import torchvision.transforms as T
from decord import VideoReader, cpu
from PIL import Image
from common.registry import registry
from torchvision.transforms.functional import InterpolationMode
from transformers import AutoModel, AutoTokenizer
import math

IMAGENET_MEAN = (0.485, 0.456, 0.406)
IMAGENET_STD = (0.229, 0.224, 0.225)

@registry.register_llm("internvl2")
class InternVL2:
    def __init__(
        self,
        engine="InternVL2-2B",
        temperature=0.1,
        max_tokens=4096,
        top_p=0.95,
        stop=[""],
        retry_delays=1,
        max_retry_iters=100,
        context_length=4096,
        system_message="",
        cuda_rank=None,
    ):
        self.engine = engine
        self.temperature = temperature
        self.max_tokens = max_tokens
        self.top_p = top_p
        self.stop = stop
        self.retry_delays = retry_delays
        self.max_retry_iters = max_retry_iters
        self.context_length = context_length
        self.system_message = system_message
        # import pdb; pdb.set_trace()
        if 'InternVL2-76B' in self.engine:
            device_map = self.split_model('InternVL2-Llama3-76B')
            print("Split model for InternVL2-Llama3-76B")
            self.model = AutoModel.from_pretrained(
                self.engine,
                torch_dtype=torch.bfloat16,
                low_cpu_mem_usage=True,
                device_map=device_map,
                attn_implementation="flash_attention_2",
                trust_remote_code=True
            ).eval()
        elif 'InternVL2-40B' in self.engine:
            device_map = self.split_model('InternVL2-40B')
            print("Split model for InternVL2-40B")
            self.model = AutoModel.from_pretrained(
                self.engine,
                torch_dtype=torch.bfloat16,
                low_cpu_mem_usage=True,
                device_map=device_map,
                attn_implementation="flash_attention_2",
                trust_remote_code=True
            ).eval()
        else:
            device_map = 'auto'
            self.model = AutoModel.from_pretrained(
                self.engine,
                torch_dtype=torch.bfloat16,
                low_cpu_mem_usage=True,
                trust_remote_code=True,
                attn_implementation="flash_attention_2",
                device_map=device_map
            ).eval()
        self.tokenizer = AutoTokenizer.from_pretrained(self.engine, trust_remote_code=True)
        self.generation_config = dict(
            max_new_tokens=self.max_tokens,
            temperature=self.temperature,
            top_p=self.top_p,
            do_sample=(self.temperature != 0),
        )


    def generate(self, conversation):
        if len(conversation[0]['content']) == 5:
            pixel_values_n = [self._load_image(_["image_url"], max_num=12).to(torch.bfloat16).to(self.model.device) for _ in conversation[0]['content'] if _['type'] == "image"]
            # pixel_values1 = self._load_image('./examples/image1.jpg', max_num=12).to(torch.bfloat16).to(self.model.device)
            # pixel_values2 = self._load_image('./examples/image2.jpg', max_num=12).to(torch.bfloat16).to(self.model.device)
            pixel_values = torch.cat([_ for _ in pixel_values_n], dim=0)
            num_patches_list = [_.size(0) for _ in pixel_values_n]
            messages = ""
            image_count = 1
            for _ in range(len(conversation[0]['content'])):
                if conversation[0]['content'][_]['type'] == "text":
                    messages += conversation[0]['content'][_]['text']
                elif conversation[0]['content'][_]['type'] == "image":
                    messages += f'Image-{image_count}: <image>'
                    image_count += 1
            response = self.model.chat(self.tokenizer, pixel_values, messages, self.generation_config,
                                        num_patches_list=num_patches_list)
        else:
            conversation = self._convert_conversation(conversation)
            pixel_values = self._load_image(conversation[1]['image'], max_num=12).to(torch.bfloat16).to(self.model.device)
            question = conversation[0]['text']
            response = self.model.chat(self.tokenizer, pixel_values, question, self.generation_config)
        return response

    def _convert_conversation(self, conversation):
        converted_conversation = []
        if len (conversation) != 1:
            print("InternVL only supports 1-turn conversation")
            raise NotImplementedError
        for message in conversation:
            for content in message["content"]:
                if content["type"] == "text":
                    converted_conversation.append({
                        'text': content["text"]
                    })
                elif content["type"] == "image":
                    converted_conversation.append({
                        'image': content["image_url"]
                    })              
                else:
                    raise NotImplementedError
        return converted_conversation
        
    def split_model(self, model_name):
        device_map = {}
        world_size = torch.cuda.device_count()
        num_layers = {'InternVL2-8B': 32, 'InternVL2-26B': 48,
                      'InternVL2-40B': 60, 'InternVL2-Llama3-76B': 80}[model_name]
        # Since the first GPU will be used for ViT, treat it as half a GPU.
        num_layers_per_gpu = math.ceil(num_layers / (world_size - 0.5))
        num_layers_per_gpu = [num_layers_per_gpu] * world_size
        num_layers_per_gpu[0] = math.ceil(num_layers_per_gpu[0] * 0.5)
        layer_cnt = 0
        for i, num_layer in enumerate(num_layers_per_gpu):
            for j in range(num_layer):
                device_map[f'language_model.model.layers.{layer_cnt}'] = i
                layer_cnt += 1
        device_map['vision_model'] = 0
        device_map['mlp1'] = 0
        device_map['language_model.model.tok_embeddings'] = 0
        device_map['language_model.model.embed_tokens'] = 0
        device_map['language_model.output'] = 0
        device_map['language_model.model.norm'] = 0
        device_map['language_model.lm_head'] = 0
        device_map[f'language_model.model.layers.{num_layers - 1}'] = 0
    
        return device_map

    def _build_transform(self, input_size):
        MEAN, STD = IMAGENET_MEAN, IMAGENET_STD
        transform = T.Compose([
            T.Lambda(lambda img: img.convert('RGB') if img.mode != 'RGB' else img),
            T.Resize((input_size, input_size), interpolation=InterpolationMode.BICUBIC),
            T.ToTensor(),
            T.Normalize(mean=MEAN, std=STD)
        ])
        return transform


    def _find_closest_aspect_ratio(self, aspect_ratio, target_ratios, width, height, image_size):
        best_ratio_diff = float('inf')
        best_ratio = (1, 1)
        area = width * height
        for ratio in target_ratios:
            target_aspect_ratio = ratio[0] / ratio[1]
            ratio_diff = abs(aspect_ratio - target_aspect_ratio)
            if ratio_diff < best_ratio_diff:
                best_ratio_diff = ratio_diff
                best_ratio = ratio
            elif ratio_diff == best_ratio_diff:
                if area > 0.5 * image_size * image_size * ratio[0] * ratio[1]:
                    best_ratio = ratio
        return best_ratio


    def _dynamic_preprocess(self, image, min_num=1, max_num=6, image_size=448, use_thumbnail=False):
        orig_width, orig_height = image.size
        aspect_ratio = orig_width / orig_height

        # calculate the existing image aspect ratio
        target_ratios = set(
            (i, j) for n in range(min_num, max_num + 1) for i in range(1, n + 1) for j in range(1, n + 1) if
            i * j <= max_num and i * j >= min_num)
        target_ratios = sorted(target_ratios, key=lambda x: x[0] * x[1])

        # find the closest aspect ratio to the target
        target_aspect_ratio = self._find_closest_aspect_ratio(
            aspect_ratio, target_ratios, orig_width, orig_height, image_size)

        # calculate the target width and height
        target_width = image_size * target_aspect_ratio[0]
        target_height = image_size * target_aspect_ratio[1]
        blocks = target_aspect_ratio[0] * target_aspect_ratio[1]

        # resize the image
        resized_img = image.resize((target_width, target_height))
        processed_images = []
        for i in range(blocks):
            box = (
                (i % (target_width // image_size)) * image_size,
                (i // (target_width // image_size)) * image_size,
                ((i % (target_width // image_size)) + 1) * image_size,
                ((i // (target_width // image_size)) + 1) * image_size
            )
            # split the image
            split_img = resized_img.crop(box)
            processed_images.append(split_img)
        assert len(processed_images) == blocks
        if use_thumbnail and len(processed_images) != 1:
            thumbnail_img = image.resize((image_size, image_size))
            processed_images.append(thumbnail_img)
        return processed_images


    def _load_image(self, image_file, input_size=448, max_num=12):
        image = Image.open(image_file).convert('RGB')
        transform = self._build_transform(input_size=input_size)
        images = self._dynamic_preprocess(image, image_size=input_size, use_thumbnail=True, max_num=max_num)
        pixel_values = [transform(image) for image in images]
        pixel_values = torch.stack(pixel_values)
        return pixel_values


    @classmethod
    def from_config(cls, config):
        engine = config.get("engine", "gpt-35-turbo")
        temperature = config.get("temperature", 0)
        max_tokens = config.get("max_tokens", 100)
        system_message = config.get("system_message", "")
        top_p = config.get("top_p", 1)
        stop = config.get("stop", ["\n"])
        retry_delays = config.get("retry_delays", 10)
        context_length = config.get("context_length", 4096)
        cuda_rank = config.get("cuda_rank", None)
        return cls(
            engine=engine,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            retry_delays=retry_delays,
            system_message=system_message,
            context_length=context_length,
            stop=stop,
            cuda_rank=cuda_rank,
        )
