
# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt
from matplotlib import patches

# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

np.random.seed(0)

# New domain data: Literature
features = [
    "Plot structure",
    "Character development",
    "Writing style",
    "Pacing",
    "Themes",
    "Symbolism",
    "Dialogue",
    "Setting",
    "Narrative technique",
    "World-building",
    "Emotional impact",
]
impact_values = [0.07, -0.02, 0.05, -0.01, 0.03, -0.04, 0.06, -0.05, 0.08, -0.03, 0.04]
# Starting x-axis value
start_x = 0.50
cumulative_values = [start_x]

axvhline = 0.60
ylim = [-0.5, 11]
xlim = [0.4, 0.7]
xticks = [0.45, 0.50, 0.55, 0.60, 0.65]
textposition = [[0.60, 10.5], [0.50, -2]]
textlabels = ["Overall Impact = 0.60", "Initial Value = 0.50"]
xlabel = "Impact on Literary Appreciation"
ylabel = "Literary Features"
title = "Impact of Literary Features on Appreciation"
fig_suptitle = "Literary Impact Analysis"

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
# Calculate the start point for each bar
colors_positive = "#1f77b4"  # Blue shades for positive impact
colors_negative = "#ff7f0e"  # Orange shades for negative impact

for impact in impact_values:
    next_value = cumulative_values[-1] + impact
    cumulative_values.append(next_value)

# Define colors for positive and negative impact
colors = [colors_positive if impact > 0 else colors_negative for impact in impact_values]

fig, ax = plt.subplots(figsize=(10, 8))
# Add dashed lines to connect bars
for i in range(len(impact_values) - 1):
    plt.plot(
        [cumulative_values[i + 1], cumulative_values[i + 1]],
        [features[i], features[i + 1]],
        "k--",
        linewidth=0.5,
        color="gray",
    )
# Customize bars with arrows
for idx, (feature, impact) in enumerate(zip(features, impact_values)):
    left = cumulative_values[idx]
    bar_width = 0.6  # Set the bar width

    if impact > 0:
        color = colors_positive
        width = impact
        text_x = left + width
        # Draw the main bar body
        ax.add_patch(
            patches.Rectangle(
                (left, idx - bar_width / 2),
                width - 0.005,
                bar_width,
                facecolor=color,
            )
        )
        # Draw the tip
        ax.add_patch(
            patches.Polygon(
                [
                    (left + width - 0.005, idx - bar_width / 2),
                    (left + width, idx),
                    (left + width - 0.005, idx + bar_width / 2),
                ],
                closed=True,
                facecolor=color,
            )
        )
        ax.text(text_x, idx, f"+{impact}", va="center", ha="left", color=color)
    else:
        color = colors_negative
        width = -impact  # Width is positive for drawing
        text_x = left - width
        if impact == 0:
            ax.add_patch(
                patches.Rectangle(
                    (left - width, idx - bar_width / 2),
                    width,
                    bar_width,
                    facecolor=color,
                    edgecolor=color,
                )
            )
        else:
            # Draw the main bar body
            ax.add_patch(
                patches.Rectangle(
                    (left - width + 0.005, idx - bar_width / 2),
                    width - 0.005,
                    bar_width,
                    facecolor=color,
                )
            )
            # Draw the tip
            ax.add_patch(
                patches.Polygon(
                    [
                        (left - width + 0.005, idx - bar_width / 2),
                        (left - width, idx),
                        (left - width + 0.005, idx + bar_width / 2),
                    ],
                    closed=True,
                    facecolor=color,
                )
            )

        ax.text(
            text_x, idx, f"-{np.abs(impact)}", va="center", ha="right", color=color
        )

ax.axvline(axvhline, linestyle="--", color="gray", alpha=0.3)
# Set labels and title
ax.yaxis.grid(True, linestyle="dotted", color="gray", alpha=0.3)
ax.set_axisbelow(True)
ax.tick_params(axis="both", length=0)
# Remove spines
ax.spines["right"].set_visible(False)
ax.spines["top"].set_visible(False)
ax.spines["left"].set_visible(False)
ax.set_ylim(ylim)
ax.set_xlim(xlim)
ax.set_xticks(xticks)
ax.set_xlabel(xlabel)
ax.set_ylabel(ylabel)
ax.set_title(title)
fig.suptitle(fig_suptitle)
# Add text annotation for initial value and policy impact
ax.text(
    textposition[0][0],
    textposition[0][1],
    textlabels[0],
    va="bottom",
    ha="center",
    color="grey",
)
ax.text(
    textposition[1][0],
    textposition[1][1],
    textlabels[1],
    va="bottom",
    ha="center",
    color="grey",
)

# ===================
# Part 4: Saving Output
# ===================
plt.tight_layout()
plt.savefig("HR_59.pdf", bbox_inches="tight")
