
# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt

# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

np.random.seed(0)

# Sample health domain data: Resting heart rates for two age groups
young_adults_heart_rate = np.random.normal(70, 8, 200)
older_adults_heart_rate = np.random.normal(75, 10, 200)

# Data and configuration for the plot
data = [young_adults_heart_rate, older_adults_heart_rate]
categories = ["Young Adults", "Older Adults"]
ylabel = "Resting Heart Rate (bpm)"
xticks = [1, 2]
ylim = [40, 110]
title = "Resting Heart Rates - Young Adults vs. Older Adults"

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
fig, ax = plt.subplots(figsize=(8, 6))  # Adjusting figure size
violin_parts = ax.violinplot(data, showmeans=False, showmedians=True, showextrema=False)

# Customizing the appearance
ax.set_title(title)
ax.set_ylabel(ylabel)
ax.set_xticks(xticks)
ax.set_xticklabels(categories)
ax.grid(axis="y", linestyle="--", alpha=0.7)  # Adding dashed horizontal grid lines
ax.set_ylim(ylim)  # Setting y-axis limits

# Removing the ticks on the x and y axes
ax.tick_params(axis="x", which="both", length=0)  # Remove x-axis ticks
ax.tick_params(axis="y", which="both", length=0)  # Remove y-axis ticks

# Coloring the violins and adding the desired statistical annotations
for i, (pc, d) in enumerate(zip(violin_parts["bodies"], data)):
    pc.set_facecolor(["#20b2aa", "#ff7f50"][i])  # Teal and Coral colors
    pc.set_edgecolor("black")
    pc.set_alpha(0.8)

    # Calculate the quartiles and interquartile range
    quartile1, median, quartile3 = np.percentile(d, [25, 50, 75])
    iqr = quartile3 - quartile1

    # Calculate lower and upper whiskers using 1.5xIQR rule
    lower_whisker = np.min(d[d >= quartile1 - 1.5 * iqr])
    upper_whisker = np.max(d[d <= quartile3 + 1.5 * iqr])

    # Placing lines for median, quartiles, and whiskers
    ax.vlines(i + 1, quartile1, quartile3, color="k", linestyle="-", lw=4)
    ax.scatter(i + 1, median, color="w", s=30, zorder=3)
    ax.vlines(i + 1, lower_whisker, upper_whisker, color="k", linestyle="--", lw=1.5)

# Remove the lines (medians, whiskers, etc.)
for partname in ("cbars", "cmins", "cmaxes", "cmedians"):
    vp = violin_parts.get(partname)
    if vp:
        vp.set_visible(False)

# ===================
# Part 4: Saving Output
# ===================
# Adjust layout for better fit
plt.tight_layout()

# Display the plot
plt.savefig("violin_37.pdf", bbox_inches="tight")
