

# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt
from scipy.stats import gaussian_kde


# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

np.random.seed(0)
import random

# Updated for a new domain: Popularity of Different Psychological Research Areas
data = np.random.beta(a=[4, 5, 6, 3, 7], b=[6, 4, 5, 7, 4], size=(10, 5))
data_memory = np.random.beta(a=[5, 3, 4, 6, 2], b=[5, 6, 7, 5, 8], size=(40, 5))

xticklabels = ["Cognitive", "Behavioral", "Developmental", "Social", "Clinical"]
legend_labels = ["Current Studies", "Historical Studies"]
scaling_factor = 1
violin_width = 0.5
xlabel = "Psychological Research Area"
ylabel = "Study Frequency"

# Adjust the offsets for 5 groups
offsets = np.linspace(-3, 3, 5)

title = "Study Frequency in Different Psychological Research Areas"

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
# Set the figure size
fig, ax = plt.subplots(figsize=(10, 6))  # Increased width for better spacing

# Define the colors for each group
colors = ["#004c6d", "#37718e"]  # Deep Blue and Light Blue

# Plot the half-violins with an offset for 5 groups
for i in range(data.shape[1]):
    offset = offsets[i]

    # Plot data without memory
    kde_data = gaussian_kde(data[:, i])
    kde_x = np.linspace(0, 1, 300)
    kde_data_y = kde_data(kde_x)
    kde_data_y_scaled = kde_data_y / max(kde_data_y) * violin_width
    ax.fill_betweenx(
        kde_x,
        kde_data_y_scaled * scaling_factor + offset,
        offset,
        color=colors[0],
        edgecolor="black",
        alpha=0.7
    )

    # Plot data with memory
    kde_data_memory = gaussian_kde(data_memory[:, i])
    kde_data_memory_y = kde_data_memory(kde_x)
    kde_data_memory_y_scaled = kde_data_memory_y / max(kde_data_memory_y) * violin_width
    ax.fill_betweenx(
        kde_x,
        offset,
        -kde_data_memory_y_scaled * scaling_factor + offset,
        color=colors[1],
        edgecolor="black",
        alpha=0.7
    )

    # Add yellow stars at the top of each violin plot for emphasis
    ax.scatter(
        offset,
        random.uniform(0.2, 0.8),
        marker="*",
        color="yellow",
        s=260,
        zorder=3,
        edgecolor="black",
    )

# Set axis labels and limits
ax.set_xlabel(xlabel, fontsize=14, fontweight='bold')
ax.set_ylabel(ylabel, fontsize=14, fontweight='bold')
ax.set_xlim(
    min(offsets) - scaling_factor - violin_width,
    max(offsets) + scaling_factor + violin_width,
)
ax.set_xticks(offsets)
ax.set_xticklabels(xticklabels, fontsize=12)

# Add title
ax.set_title(title, fontsize=16, fontweight='bold')

# Adjust the legend
handles = [
    plt.Rectangle((0, 0), 1, 1, color=color, edgecolor="black")
    for color in colors
]
ax.legend(handles, legend_labels, loc="upper left", ncol=1)

# ===================
# Part 4: Saving Output
# ===================
# Adjust layout and save the plot
plt.tight_layout()
plt.savefig("violin_73.pdf", bbox_inches="tight")

