# ===================
# Part 1: Importing Libraries
# ===================

import matplotlib.pyplot as plt

# ===================
# Part 2: Data Preparation
# ===================
import networkx as nx

# Create a directed graph
G = nx.DiGraph()

# Add nodes representing different entities in the healthcare information flow
nodes = ['Central Hospital', 'Clinic A', 'Clinic B', 'Pharmacy A', 'Patient X']
for node in nodes:
    G.add_node(node)

# Define edges representing the information flow between entities
edges = [
    ('Central Hospital', 'Clinic A'),
    ('Central Hospital', 'Clinic B'),
    ('Clinic A', 'Pharmacy A'),
    ('Clinic B', 'Patient X'),
    ('Clinic A', 'Clinic A'),  # Self-loop for internal data processing in Clinic A
    ('Clinic B', 'Clinic B'),  # Self-loop for internal data processing in Clinic B
    ('Central Hospital', 'Central Hospital')  # Self-loop for system monitoring or internal feedback
]

# Add edges to the graph
G.add_edges_from(edges)

# Define positions for a clear layout
pos = nx.spring_layout(G, seed=42)

title = 'Healthcare Information Flow Between Entities'

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
plt.figure(figsize=(12, 10))

# Draw the graph with customized node colors and labels
node_colors = ['green' if 'Central Hospital' in node else 'blue' if 'Clinic A' in node else 'yellow' for node in G.nodes()]
nx.draw(G, pos, with_labels=True, node_color=node_colors, node_size=3000, font_size=10, font_weight='bold', edge_color='gray')

# Draw edges with differentiated styles for self-loops
self_loops = [(u, v) for u, v in G.edges() if u == v]
other_edges = [(u, v) for u, v in G.edges() if u != v]

nx.draw_networkx_edges(G, pos, edgelist=other_edges, arrowstyle='-|>', arrowsize=20, width=2.0, edge_color='black')
nx.draw_networkx_edges(G, pos, edgelist=self_loops, arrowstyle='-|>', style='dashed', arrowsize=20, width=2.0, edge_color='red')

# Title and labels
plt.title(title, fontsize=15)
plt.axis('off')

# ===================
# Part 4: Saving Output
# ===================
# Displaying the plot with tight layout to minimize white space
plt.tight_layout()
plt.savefig("graph_14.pdf", bbox_inches="tight")