import matplotlib.pyplot as plt
import numpy as np; np.random.seed(0); np.random.seed(0)

import matplotlib
import matplotlib as mpl

# ===================
# Part 2: Data Preparation
# ===================
devices = ["Laptop", "Smartphone", "Tablet", "Smartwatch", "Router", "Printer", "Monitor"]
brands = [
    "TechCorp",
    "Innovate Ltd.",
    "GadgetWorks",
    "FutureTech",
    "DeviceHub",
    "ElectroGoods",
    "DigitalWorld",
]

sales = np.array(
    [
        [120, 450, 230, 300, 150, 500, 400],
        [340, 290, 520, 310, 270, 150, 200],
        [180, 300, 150, 450, 310, 270, 290],
        [200, 220, 130, 150, 410, 230, 190],
        [160, 340, 180, 360, 390, 520, 270],
        [310, 240, 220, 150, 160, 430, 480],
        [130, 290, 180, 210, 140, 380, 500],
    ]
)

# ===================
# Part 3: Plot Configuration and Rendering
# ===================


def heatmap(
    data, row_labels, col_labels, ax=None, cbar_kw=None, cbarlabel="", **kwargs
):
    if ax is None:
        ax = plt.gca()

    if cbar_kw is None:
        cbar_kw = {}

    # Plot the heatmap
    im = ax.imshow(data, **kwargs)

    # Create colorbar
    cbar = ax.figure.colorbar(im, ax=ax, **cbar_kw)
    cbar.ax.set_ylabel(cbarlabel, rotation=-90, va="bottom")

    # Show all ticks and label them with the respective list entries.
    ax.set_xticks(np.arange(data.shape[1]), labels=col_labels)
    ax.set_yticks(np.arange(data.shape[0]), labels=row_labels)

    # Let the horizontal axes labeling appear on top.
    ax.tick_params(top=True, bottom=False, labeltop=True, labelbottom=False)

    # Rotate the tick labels and set their alignment.
    plt.setp(ax.get_xticklabels(), rotation=-30, ha="right", rotation_mode="anchor")

    # Turn spines off and create white grid.
    ax.spines[:].set_visible(False)

    ax.set_xticks(np.arange(data.shape[1] + 1) - 0.5, minor=True)
    ax.set_yticks(np.arange(data.shape[0] + 1) - 0.5, minor=True)
    ax.grid(which="minor", color="w", linestyle="-", linewidth=3)
    ax.tick_params(which="minor", bottom=False, left=False)

    return im, cbar


def annotate_heatmap(
    im,
    data=None,
    valfmt="{x:.2f}",
    textcolors=("black", "white"),
    threshold=None,
    **textkw,
):
    if not isinstance(data, (list, np.ndarray)):
        data = im.get_array()

    # Normalize the threshold to the images color range.
    if threshold is not None:
        threshold = im.norm(threshold)
    else:
        threshold = im.norm(data.max()) / 2.0

    # Set default alignment to center, but allow it to be
    # overwritten by textkw.
    kw = dict(horizontalalignment="center", verticalalignment="center")
    kw.update(textkw)

    # Get the formatter in case a string is supplied
    if isinstance(valfmt, str):
        valfmt = matplotlib.ticker.StrMethodFormatter(valfmt)

    # Loop over the data and create a `Text` for each "pixel".
    # Change the text's color depending on the data.
    texts = []
    for i in range(data.shape[0]):
        for j in range(data.shape[1]):
            kw.update(color=textcolors[int(im.norm(data[i, j]) > threshold)])
            text = im.axes.text(j, i, valfmt(data[i, j], None), **kw)
            texts.append(text)

    return texts


fig, ((ax, ax2), (ax3, ax4)) = plt.subplots(2, 2, figsize=(8, 6))

# Replicate the above example with a different font size and colormap.
im, _ = heatmap(
    sales, devices, brands, ax=ax, cmap="Wistia", cbarlabel="sales [units/month]"
)
annotate_heatmap(im, valfmt="{x:.1f}", size=7)

# Create some new data, give further arguments to imshow (vmin),
# use an integer format on the annotations and provide some colors.
data = np.random.randint(20, 500, size=(7, 7))
y = [f"Day {i}" for i in range(1, 8)]
x = [f"Site {i}" for i in list("ABCDEFG")]
im, _ = heatmap(
    data, y, x, ax=ax2, vmin=0, cmap="magma_r", cbarlabel="daily visitors"
)
annotate_heatmap(im, valfmt="{x:d}", size=7, threshold=200, textcolors=("red", "white"))

# Sometimes even the data itself is categorical. Here we use a
# `matplotlib.colors.BoundaryNorm` to get the data into classes
# and use this to colorize the plot, but also to obtain the class
# labels from an array of classes.
data = np.random.randn(6, 6)
y = [f"Quarter {i}" for i in range(1, 7)]
x = [f"Year {i}" for i in range(2018, 2024)]

ratings = list("ABCDEFG")
norm = matplotlib.colors.BoundaryNorm(np.linspace(-3.5, 3.5, 8), 7)
fmt = matplotlib.ticker.FuncFormatter(lambda x, pos: ratings[::-1][norm(x)])

im, _ = heatmap(
    data,
    y,
    x,
    ax=ax3,
    cmap=mpl.colormaps["PiYG"].resampled(7),
    norm=norm,
    cbar_kw=dict(ticks=np.arange(-3, 4), format=fmt),
    cbarlabel="Satisfaction Rating",
)

annotate_heatmap(
    im, valfmt=fmt, size=9, fontweight="bold", threshold=-1, textcolors=("red", "black")
)

# We can nicely plot a correlation matrix. Since this is bound by -1 and 1,
# we use those as vmin and vmax. We may also remove leading zeros and hide
# the diagonal elements (which are all 1) by using a
# `matplotlib.ticker.FuncFormatter`.
corr_matrix = np.corrcoef(sales)
im, _ = heatmap(
    corr_matrix,
    devices,
    devices,
    ax=ax4,
    cmap="PuOr",
    vmin=-1,
    vmax=1,
    cbarlabel="correlation coeff.",
)


def func(x, pos):
    return f"{x:.2f}".replace("0.", ".").replace("1.00", "")


annotate_heatmap(im, valfmt=matplotlib.ticker.FuncFormatter(func), size=7)

# ===================
# Part 4: Saving Output
# ===================
plt.tight_layout()
plt.savefig('heatmap_25.pdf', bbox_inches='tight')
