
# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt
from matplotlib import patches

# ===================
# Part 2: Data Preparation
# ===================
# Importing numpy for data generation
import numpy as np

np.random.seed(42)

# New data for the energy domain
features = [
    "19 other features",
    "renewable energy",
    "non-renewable energy",
    "energy efficiency",
    "carbon footprint",
    "greenhouse gas emissions",
    "electricity consumption",
    "hydropower",
    "solar power",
    "wind power",
    "nuclear energy",
]

impact_values = [0, 0.12, -0.15, 0.08, -0.07, 0.10, -0.05, 0.09, 0.11, 0.04, -0.03]
# Starting x-axis value
start_x = 0.50
cumulative_values = [start_x]

axvhline = 0.76
ylim = [-0.5, 11]
xlim = [0.3, 0.9]
xticks = [0.35, 0.40, 0.45, 0.50, 0.55, 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90]
textposition = [[0.76, 10.5], [0.50, -2]]
textlabels = ["f(x) = 0.76", "f(x) = 0.50"]
xlabel = "Cumulative Impact"
ylabel = "Energy Features"
title = "Impact of Various Factors on Energy Domain"
legendlabels = ["Positive Impact", "Negative Impact"]

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
# Calculate the start point for each bar
for impact in impact_values:
    next_value = cumulative_values[-1] + impact
    cumulative_values.append(next_value)

# Define colors for positive and negative impact
colors = ["#2ca02c" if impact > 0 else "#d62728" for impact in impact_values]

fig, ax = plt.subplots(figsize=(10, 8))
# Add dashed lines to connect bars
for i in range(len(impact_values) - 1):
    plt.plot(
        [cumulative_values[i + 1], cumulative_values[i + 1]],
        [features[i], features[i + 1]],
        "k--",
        linewidth=0.5,
        color="gray",
    )

# Customize bars with arrows
for idx, (feature, impact) in enumerate(zip(features, impact_values)):
    left = cumulative_values[idx]
    bar_width = 0.6  # Set the bar width

    if impact > 0:
        color = "#2ca02c"
        width = impact
        text_x = left + width
        # Draw the main bar body
        ax.add_patch(
            patches.Rectangle(
                (left, idx - bar_width / 2),
                width - 0.005,
                bar_width,
                facecolor=color,
            )
        )
        # Draw the tip
        ax.add_patch(
            patches.Polygon(
                [
                    (left + width - 0.005, idx - bar_width / 2),
                    (left + width, idx),
                    (left + width - 0.005, idx + bar_width / 2),
                ],
                closed=True,
                facecolor=color,
            )
        )
        ax.text(text_x, idx, f"+{impact}", va="center", ha="left", color=color)
    else:
        color = "#d62728"
        width = -impact  # Width is positive for drawing
        text_x = left - width
        if impact == 0:
            ax.add_patch(
                patches.Rectangle(
                    (left - width, idx - bar_width / 2),
                    width,
                    bar_width,
                    facecolor=color,
                    edgecolor=color,
                )
            )
        else:
            # Draw the main bar body
            ax.add_patch(
                patches.Rectangle(
                    (left - width + 0.005, idx - bar_width / 2),
                    width - 0.005,
                    bar_width,
                    facecolor=color,
                )
            )
            # Draw the tip
            ax.add_patch(
                patches.Polygon(
                    [
                        (left - width + 0.005, idx - bar_width / 2),
                        (left - width, idx),
                        (left - width + 0.005, idx + bar_width / 2),
                    ],
                    closed=True,
                    facecolor=color,
                )
            )

        ax.text(
            text_x, idx, f"-{np.abs(impact)}", va="center", ha="right", color=color
        )

ax.axvline(axvhline, linestyle="--", color="gray", alpha=0.3)
# Set labels and title
ax.yaxis.grid(True, linestyle="dotted", color="gray", alpha=0.3)
ax.set_axisbelow(True)
ax.tick_params(axis="both", length=0)
# Remove spines
ax.spines["right"].set_visible(False)
ax.spines["top"].set_visible(False)
ax.spines["left"].set_visible(False)
ax.set_ylim(ylim)
ax.set_xlim(xlim)
ax.set_xticks(xticks)
ax.set_xlabel(xlabel)
ax.set_ylabel(ylabel)
ax.set_title(title)
# Add text annotation for f(x)
ax.text(
    textposition[0][0],
    textposition[0][1],
    textlabels[0],
    va="bottom",
    ha="center",
    color="grey",
)
ax.text(
    textposition[1][0],
    textposition[1][1],
    textlabels[1],
    va="bottom",
    ha="center",
    color="grey",
)

# ===================
# Part 4: Saving Output
# ===================
plt.tight_layout()
plt.savefig("HR_61.pdf", bbox_inches="tight")
