
# ===================
# Part 1: Importing Libraries
# ===================
import networkx as nx
import matplotlib.pyplot as plt

# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

# Create a directed graph for environmental elements
G = nx.DiGraph()
# Adding nodes representing environmental elements
G.add_nodes_from(
    [
        ("Atmosphere", {"type": "element"}),
        ("Hydrosphere", {"type": "element"}),
        ("Biosphere", {"type": "element"}),
        ("Lithosphere", {"type": "element"}),
    ]
)

# Adding edges representing interactions
G.add_edges_from(
    [
        ("Atmosphere", "Hydrosphere"),
        ("Hydrosphere", "Biosphere"),
        ("Biosphere", "Lithosphere"),
        ("Lithosphere", "Atmosphere"),
    ]
)

# Adding self-loops representing internal processes
self_loops = [
    ("Atmosphere", "Atmosphere"),
    ("Hydrosphere", "Hydrosphere"),
    ("Biosphere", "Biosphere"),
    ("Lithosphere", "Lithosphere"),
]
G.add_edges_from(self_loops)

# Positioning nodes in a circular layout
pos = nx.circular_layout(G)

labels = ["Inter-element Interactions", "Internal Element Processes"]

title = "Interactions Between Environmental Elements"

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
plt.figure(figsize=(10, 8))

# Draw nodes and edges with base settings
color_map = {
    "Atmosphere": "skyblue",
    "Hydrosphere": "deepskyblue",
    "Biosphere": "seagreen",
    "Lithosphere": "sienna",
}
node_colors = [color_map[node] for node in G.nodes()]

nx.draw(
    G,
    pos,
    with_labels=True,
    node_color=node_colors,
    node_size=3000,
    edge_color="lightgray",
    linewidths=1.5,
    font_size=12,
    font_weight="bold",
)

# Highlight self-loops with a different style
nx.draw_networkx_edges(
    G,
    pos,
    edgelist=self_loops,
    edge_color="forestgreen",
    style="dashed",
    arrowstyle="<|-",
    arrowsize=20,
    width=2,
)

# Adding title and legend
plt.title(title, fontsize=16)
edge_legend = plt.Line2D([], [], linestyle="solid", color="lightgray", label=labels[0])
self_loop_legend = plt.Line2D(
    [], [], linestyle="dashed", color="forestgreen", label=labels[1]
)
plt.legend(handles=[edge_legend, self_loop_legend])

# ===================
# Part 4: Saving Output
# ===================
# Displaying the plot with tight layout to minimize white space
plt.tight_layout()
plt.savefig("graph_58.pdf", bbox_inches="tight")
