
# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt
from matplotlib.colors import LogNorm

# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

np.random.seed(42)

# Defining the colormap from light green to dark green
cmap = plt.cm.Greens

# Sample data: a 5x6 grid, similar to the provided heatmap, for media consumption preferences
data = np.array(
    [
        [30.2, 25.0, 5.9, 10.0, 15.6, 13.3],
        [22.1, 35.3, 8.1, 12.5, 18.4, 3.6],
        [40.1, 20.6, 7.5, 5.8, 19.2, 6.8],
        [15.4, 30.8, 12.9, 10.4, 20.5, 10.3],
        [18.7, 25.4, 11.3, 15.7, 21.8, 7.1],
    ]
)

# X and Y labels
xticklabels = ["TV", "Radio", "Internet", "Newspaper", "Social Media", "Other"]
yticklabels = ["Age 18-24", "Age 25-34", "Age 35-44", "Age 45-54", "Age 55+"]
x_label = "Media Type"
y_label = "Age Group"

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
# Set up the figure and axis
fig, ax = plt.subplots(figsize=(10, 8))

# Set up the colormap and norm (log scale)
norm = LogNorm(vmin=0.1, vmax=50)

# Create the scatter plot
for i in range(data.shape[0]):
    for j in range(data.shape[1]):
        # Calculate the color based on the original value
        color = cmap(norm(data[i, j]))

        # Draw the circle with a fixed size
        circle = plt.Circle((j, i), 0.4, color=color)  
        ax.add_artist(circle)

        # Determine text color based on the value
        text_color = "white" if data[i, j] > 20 else "black"

        # Add the text inside the circle
        ax.text(j, i, f"{data[i, j]:.1f}%", ha="center", va="center", color=text_color)

# Set labels for x and y axes
ax.set_xticks(range(len(xticklabels)))
ax.set_xticklabels(xticklabels, ha="center")
ax.set_yticks(range(len(yticklabels)))
ax.set_yticklabels(yticklabels, va="center")

# Adding titles for the axes
ax.set_xlabel(x_label, fontsize=14)
ax.set_ylabel(y_label, fontsize=14)

# Set the limits of the axes; they should be one more than your data range
ax.set_xlim(-0.5, data.shape[1] - 0.5)
ax.set_ylim(-0.5, data.shape[0] - 0.5)

# Set the aspect of the plot to be equal and add a frame
ax.set_aspect("equal")
for spine in ax.spines.values():
    spine.set_visible(True)

# Create a colorbar
sm = plt.cm.ScalarMappable(cmap=cmap, norm=norm)
sm.set_array([])
cbar = plt.colorbar(sm, ax=ax, ticks=[0.1, 1, 10, 50], orientation="vertical")
cbar.ax.set_yticklabels(["0.1", "1", "10", "50"])

# Add gridlines
plt.grid(True, which="both", color="gray", linestyle="--", linewidth=0.5)

# ===================
# Part 4: Saving Output
# ===================
# Adjust layout to fit the figure size
plt.tight_layout()

# Show the plot
plt.savefig("heatmap_91.pdf", bbox_inches="tight")
