
# ===================
# Part 1: Importing Libraries
# ===================
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D

# ===================
# Part 2: Data Preparation
# ===================
import numpy as np

np.random.seed(42)
# Sample data for societal context: Income Disparity Decrease vs. Education Level
education_levels = np.arange(1, 6)
group_1_data = np.random.rand(5, 100) * 50  # Random data, scaled to fit plausible trends
group_2_data = np.random.rand(5, 100) * 50

titles = [
    "% Decrease in Income Disparity vs. Education Level for Group 1",
    "% Decrease in Income Disparity vs. Education Level for Group 2"
]
ylims = [[0, 60], [0, 60]]
xlabel = "Education Level (1: Elementary, 5: University)"
ytickslabels = [
    ["0%", "10%", "20%", "30%", "40%", "50%", "60%"],
    ["0%", "10%", "20%", "30%", "40%", "50%", "60%"],
]
xticklabel = ["1", "2", "3", "4", "5"]

# ===================
# Part 3: Plot Configuration and Rendering
# ===================
# Create figure and subplots
fig, axs = plt.subplots(2, 1, figsize=(10, 10))
# Define the colors for the violin plots
color_group_1 = "#008080"  # Teal
color_group_2 = "#FF7F50"  # Coral

# Function to set the color of the violin plot
def set_violin_color(violin, color):
    for body in violin["bodies"]:
        body.set_facecolor(color)
        body.set_edgecolor(color)
    # Set color for the median line
    violin["cmedians"].set_color(color)

# Top subplot: Group 1 data
for i, level in enumerate(education_levels):
    vl = axs[0].violinplot(
        group_1_data[i],
        positions=[level - 0.2],
        showmedians=True,
        widths=0.3,
        showextrema=False,
    )
    set_violin_color(vl, color_group_1)

    vl = axs[0].violinplot(
        group_2_data[i],
        positions=[level + 0.2],
        showmedians=True,
        widths=0.3,
        showextrema=False,
    )
    set_violin_color(vl, color_group_2)

    axs[0].text(
        level - 0.35,
        np.median(group_1_data[i]),
        f"{int(np.median(group_1_data[i]))}%",
        ha="right",
        va="bottom",
        color=color_group_1,
    )
    axs[0].text(
        level + 0.35,
        np.median(group_2_data[i]),
        f"{int(np.median(group_2_data[i]))}%",
        ha="left",
        va="bottom",
        color=color_group_2,
    )

axs[0].set_title(titles[0])
axs[0].set_xticks(education_levels)
axs[0].set_ylim(ylims[0])
axs[0].set_xticklabels([])
axs[0].set_yticklabels(ytickslabels[0])

# Bottom subplot: Group 2 data
for i, level in enumerate(education_levels):
    vl = axs[1].violinplot(
        group_1_data[i],
        positions=[level - 0.2],
        showmedians=True,
        widths=0.3,
        showextrema=False,
    )
    set_violin_color(vl, color_group_1)

    vl = axs[1].violinplot(
        group_2_data[i],
        positions=[level + 0.2],
        showmedians=True,
        widths=0.3,
        showextrema=False,
    )
    set_violin_color(vl, color_group_2)

    axs[1].text(
        level - 0.35,
        np.median(group_1_data[i]),
        f"{int(np.median(group_1_data[i]))}%",
        ha="right",
        va="bottom",
        color=color_group_1,
    )
    axs[1].text(
        level + 0.35,
        np.median(group_2_data[i]),
        f"{int(np.median(group_2_data[i]))}%",
        ha="left",
        va="bottom",
        color=color_group_2,
    )

axs[1].set_title(titles[1])
axs[1].set_xticks(education_levels)
axs[1].set_ylim(ylims[1])
axs[1].set_xlabel(xlabel)
axs[1].set_xticklabels(xticklabel)
axs[1].set_yticklabels(ytickslabels[1])

# Use line to create a custom legend
legend_elements = [
    Line2D([0], [0], color=color_group_1, lw=2, label="Group 1"),
    Line2D([0], [0], color=color_group_2, lw=2, label="Group 2"),
]
axs[0].legend(handles=legend_elements, loc="upper right")
axs[1].legend(handles=legend_elements, loc="upper right")

# ===================
# Part 4: Saving Output
# ===================
# Adjust layout and save plot
plt.tight_layout()
plt.savefig("violin_41.pdf", bbox_inches="tight")

